@@ -45,7 +45,7 @@ void Psyco_DefineMeta(void* c_function, void* psyco_function)
 			return;
 	}
 	if (c_function == NULL) {
-		debug_printf(2, ("init: C function pointer NULL in CPython\n"));
+		debug_printf(1, ("init: C function pointer NULL in CPython\n"));
 		return;
 	}
 	key = PyInt_FromLong((long) c_function);
@@ -1123,9 +1123,27 @@ static PyObject* load_global(PsycoObject* po, PyObject* key, int next_instr)
 	   deleted from the globals in the meantime. This is why we
 	   cannot just do that in the "found in globals()" branch
 	   below. */
-	if (detect_respawn(po)) {
+	if (detect_respawn_ex(po)) {
 		/* respawning was triggered by the call to
 		   psyco_prepare_respawn_ex() below */
+		/* dictitem_check_change can use NEED_CC and
+		   NEED_FREE_REG, further updating the state of po before
+		   the actual jump to the respawn buffer.  We emulate this
+		   effect by calling dictitem_check_change() again here. */
+		static PyDictObject* dummy_dict = NULL;
+		static PyDictEntry* dummy_entry = NULL;
+		if (dummy_entry == NULL) {
+			dummy_dict = (PyDictObject*) PyDict_New(); /* immortal */
+			if (!dummy_dict || PyDict_SetItem(
+			     (PyObject*) dummy_dict, Py_None, Py_None))
+				OUT_OF_MEMORY();
+			dummy_entry = (dummy_dict->ma_lookup)(
+				dummy_dict, Py_None, PyObject_Hash(Py_None));
+			extra_assert(dummy_entry != NULL);
+		}
+		dictitem_check_change(po, dummy_dict, dummy_entry);
+		/* end of respawning -- this dummy code will now be trashed */
+		psyco_respawn_detected(po);
 		mark_varying(po, key);
 		return NULL;
 	}
@@ -1301,6 +1319,7 @@ static vinfo_t* psyco_apply_slice(PsycoObject* po, vinfo_t* u,
 					    CfReturnRef|CfPyErrIfNull,
 					    modes, v, w, NULL);
 		if (vslice != NULL) {
+			Psyco_AssertType(po, vslice, &PySlice_Type);
 			u = PsycoObject_GetItem(po, u, vslice);
 			vinfo_decref(vslice, po);
 			return u;
@@ -1365,6 +1384,7 @@ static bool psyco_assign_slice(PsycoObject* po, vinfo_t* u,
 					    CfReturnRef|CfPyErrIfNull,
 					    modes, v, w, NULL);
 		if (vslice != NULL) {
+			Psyco_AssertType(po, vslice, &PySlice_Type);
 			ok = PsycoObject_SetItem(po, u, vslice, x);
 			vinfo_decref(vslice, po);
 			return ok;
@@ -1961,6 +1981,62 @@ static PyObject* cimpl_import_name(PyObject* globals, PyObject* name,
 	return x;
 }
 
+static int cimpl_import_all_from(PyObject *locals, PyObject *v)
+{
+	PyObject *all = PyObject_GetAttrString(v, "__all__");
+	PyObject *dict, *name, *value;
+	int skip_leading_underscores = 0;
+	int pos, err;
+
+	if (all == NULL) {
+		if (!PyErr_ExceptionMatches(PyExc_AttributeError))
+			return -1; /* Unexpected error */
+		PyErr_Clear();
+		dict = PyObject_GetAttrString(v, "__dict__");
+		if (dict == NULL) {
+			if (!PyErr_ExceptionMatches(PyExc_AttributeError))
+				return -1;
+			PyErr_SetString(PyExc_ImportError,
+			"from-import-* object has no __dict__ and no __all__");
+			return -1;
+		}
+		all = PyMapping_Keys(dict);
+		Py_DECREF(dict);
+		if (all == NULL)
+			return -1;
+		skip_leading_underscores = 1;
+	}
+
+	for (pos = 0, err = 0; ; pos++) {
+		name = PySequence_GetItem(all, pos);
+		if (name == NULL) {
+			if (!PyErr_ExceptionMatches(PyExc_IndexError))
+				err = -1;
+			else
+				PyErr_Clear();
+			break;
+		}
+		if (skip_leading_underscores &&
+		    PyString_Check(name) &&
+		    PyString_AS_STRING(name)[0] == '_')
+		{
+			Py_DECREF(name);
+			continue;
+		}
+		value = PyObject_GetAttr(v, name);
+		if (value == NULL)
+			err = -1;
+		else
+			err = PyDict_SetItem(locals, name, value);
+		Py_DECREF(name);
+		Py_XDECREF(value);
+		if (err != 0)
+			break;
+	}
+	Py_DECREF(all);
+	return err;
+}
+
 
  /***************************************************************/
 /***                         Main loop                         ***/
@@ -2237,7 +2313,6 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		goto fine
 		
 	BINARY_OPCODE(BINARY_MULTIPLY, PsycoNumber_Multiply);
-	BINARY_OPCODE(BINARY_DIVIDE, PsycoNumber_Divide);
 	BINARY_OPCODE(BINARY_MODULO, PsycoNumber_Remainder);
 	BINARY_OPCODE(BINARY_ADD, PsycoNumber_Add);
 	BINARY_OPCODE(BINARY_SUBTRACT, PsycoNumber_Subtract);
@@ -2248,11 +2323,48 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 	BINARY_OPCODE(BINARY_XOR, PsycoNumber_Xor);
 	BINARY_OPCODE(BINARY_OR, PsycoNumber_Or);
 
-#ifdef BINARY_FLOOR_DIVIDE
-        BINARY_OPCODE(BINARY_FLOOR_DIVIDE, PsycoNumber_FloorDivide);
+#ifndef BINARY_FLOOR_DIVIDE
+	BINARY_OPCODE(BINARY_DIVIDE, PsycoNumber_Divide);
+	BINARY_OPCODE(INPLACE_DIVIDE, PsycoNumber_InPlaceDivide);
+#else
+	case BINARY_DIVIDE:
+		if (!_Py_QnewFlag) {
+			x = PsycoNumber_Divide(po, NTOP(2), NTOP(1));
+			if (x == NULL)
+				break;
+			POP_DECREF();
+			POP_DECREF();
+			PUSH(x);
+			goto fine;
+		}
+	/* fall through to BINARY_TRUE_DIVIDE */
         BINARY_OPCODE(BINARY_TRUE_DIVIDE, PsycoNumber_TrueDivide);
-        BINARY_OPCODE(INPLACE_FLOOR_DIVIDE, PsycoNumber_InPlaceFloorDivide);
+        BINARY_OPCODE(BINARY_FLOOR_DIVIDE, PsycoNumber_FloorDivide);
+
+	case INPLACE_DIVIDE:
+		if (!_Py_QnewFlag) {
+			x = PsycoNumber_InPlaceDivide(po, NTOP(2), NTOP(1));
+			if (x == NULL)
+				break;
+			POP_DECREF();
+			POP_DECREF();
+			PUSH(x);
+			goto fine;
+		}
+	/* fall through to INPLACE_TRUE_DIVIDE */
         BINARY_OPCODE(INPLACE_TRUE_DIVIDE, PsycoNumber_InPlaceTrueDivide);
+        BINARY_OPCODE(INPLACE_FLOOR_DIVIDE, PsycoNumber_InPlaceFloorDivide);
+#endif
+
+#ifdef LIST_APPEND
+	case LIST_APPEND:
+		w = NTOP(1);
+		v = NTOP(2);
+		if (!PsycoList_Append(po, v, w))
+			break;
+		POP_DECREF();
+		POP_DECREF();
+		goto fine;
 #endif
 
 	case INPLACE_POWER:
@@ -2267,7 +2379,6 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		goto fine;
 	
 	BINARY_OPCODE(INPLACE_MULTIPLY, PsycoNumber_InPlaceMultiply);
-	BINARY_OPCODE(INPLACE_DIVIDE, PsycoNumber_InPlaceDivide);
 	BINARY_OPCODE(INPLACE_MODULO, PsycoNumber_InPlaceRemainder);
 	BINARY_OPCODE(INPLACE_ADD, PsycoNumber_InPlaceAdd);
 	BINARY_OPCODE(INPLACE_SUBTRACT, PsycoNumber_InPlaceSubtract);
@@ -2472,6 +2583,11 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		break;
 #endif
 
+#ifdef NOP
+	case NOP:
+		goto fine;
+#endif
+
 	/*MISSING_OPCODE(YIELD_VALUE);*/
 	/*MISSING_OPCODE(EXEC_STMT);*/
 
@@ -2539,9 +2655,6 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		PUSH(x);
 		goto fine;
 
-	/*MISSING_OPCODE(STORE_NAME); -- only used in module's code objects?
-	  MISSING_OPCODE(DELETE_NAME);*/
-
 	case UNPACK_SEQUENCE:
 	{
 		int i;
@@ -2650,6 +2763,11 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		goto fine;
 	}
 
+	case STORE_NAME:
+		/* only for modules: we can only reach this point
+		   for code objects compiled from frames where
+		   f_locals == f_globals */
+		/* fall through */
 	case STORE_GLOBAL:
 	{
 		PyObject* w = GETNAMEV(oparg);
@@ -2662,6 +2780,11 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		goto fine;
 	}
 
+	case DELETE_NAME:
+		/* only for modules: we can only reach this point
+		   for code objects compiled from frames where
+		   f_locals == f_globals */
+		/* fall through */
 	case DELETE_GLOBAL:
 	{
 		PyObject* w = GETNAMEV(oparg);
@@ -2682,8 +2805,11 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		PUSH(v);
 		goto fine;
 
-	/*MISSING_OPCODE(LOAD_NAME);*/
-
+	case LOAD_NAME:
+		/* only for modules: we can only reach this point
+		   for code objects compiled from frames where
+		   f_locals == f_globals */
+		/* fall through */
 	case LOAD_GLOBAL:
 	{
 		PyObject* namev = GETNAMEV(oparg);
@@ -2698,10 +2824,10 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 				PUSH(v);
 				goto fine;
 			}
-			/* else { variable not found at all } */
+			/* else { variable not found at all,
+			   or the global has been marked as varying } */
 		}
-		/* else { Globals dict is unknown at compile-time,
-		   	  or the global has been marked as varying } */
+		/* else { Globals dict is unknown at compile-time } */
 		v = psyco_generic_call(po, cimpl_load_global,
 				       CfReturnRef|CfPyErrIfNull,
 				       "vl", LOC_GLOBALS, namev);
@@ -2852,7 +2978,20 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 		goto fine;
 	}
 	
-	/*MISSING_OPCODE(IMPORT_STAR);*/
+	case IMPORT_STAR:
+	{
+		/* only for modules: we can only reach this point
+		   for code objects compiled from frames where
+		   f_locals == f_globals */
+		x = psyco_generic_call(po, cimpl_import_all_from,
+				       CfReturnNormal|CfPyErrIfNonNull,
+				       "vv", LOC_GLOBALS, TOP());
+		if (x == NULL)
+			break;
+		vinfo_decref(x, po);
+		POP_DECREF();
+		goto fine;
+	}
 
 	case IMPORT_FROM:
 	{
@@ -3058,6 +3197,7 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 			vinfo_decref(w, po);
 		if (x == NULL)
 			break;
+		Psyco_AssertType(po, x, &PySlice_Type);
 		if (oparg == 3)
 			POP_DECREF();	/* w */
 		POP_DECREF();		/* v */
