@@ -8,6 +8,13 @@
 #define FULL_CONTROL_FLOW    1
 #define DUMP_CONTROL_FLOW    0
 
+/* the list of builtins whose usage should disable control flow
+   because they use the current frame's locals */
+static char* NoControlFlowIfBuiltin[] = {
+  "eval", "execfile", "locals", "vars", "dir", "input",
+  NULL
+};
+
 #define CONFLUENCE_TOTAL_DELAY   3   /* see comments in the code */
 #define INLINE_MAXIMUM_WEIGHT    6   /* see comments in the code */
 
@@ -59,7 +66,8 @@
 /* instructions through which it is not safe to respawn:
    LOAD_GLOBAL may produce a run-time or various compile-time values
    each time it is respawned */
-#define IS_CTXDEP_INSTR(op)  (op == LOAD_GLOBAL)
+#define IS_CTXDEP_INSTR(op)  (op == LOAD_GLOBAL ||    \
+                              op == LOAD_NAME)
 
 #define MAX_UNINTERRUPTED_RANGE   170  /* bytecode instructions */
 
@@ -69,6 +77,7 @@
                              HAS_JREL_INSTR(op) ||      \
                              HAS_JABS_INSTR(op) ||      \
                              IS_SET_LINENO(op) ||       \
+                             IS_NOP(op) ||              \
                              op == POP_TOP ||           \
                              op == ROT_TWO ||           \
                              op == ROT_THREE ||         \
@@ -86,6 +95,13 @@
                              op == BUILD_LIST ||        \
                              op == BUILD_MAP)
 
+/* opcodes that can only be compiled at module top-level,
+   when locals() is globals() */
+#define IS_MODULE_INSTR(op) (op == STORE_NAME ||        \
+                             op == DELETE_NAME ||       \
+                             op == LOAD_NAME ||         \
+                             op == IMPORT_STAR)
+
 /* all other supported instructions must be listed here. */
 #define OTHER_OPCODE(op)  (op == UNARY_POSITIVE ||            \
                            op == UNARY_NEGATIVE ||            \
@@ -105,6 +121,7 @@
                            op == BINARY_XOR ||                \
                            op == BINARY_OR ||                 \
                            IS_NEW_DIVIDE(op) ||               \
+                           IS_LIST_APPEND(op) ||              \
                            op == INPLACE_POWER ||             \
                            op == INPLACE_MULTIPLY ||          \
                            op == INPLACE_DIVIDE ||            \
@@ -193,6 +210,12 @@
 # define IS_SET_LINENO(op)       0
 #endif
 
+#ifdef NOP
+# define IS_NOP(op)             (op == NOP)
+#else
+# define IS_NOP(op)              0
+#endif
+
 #ifdef BINARY_FLOOR_DIVIDE
 # define IS_NEW_DIVIDE(op)      (op == BINARY_FLOOR_DIVIDE ||         \
                                  op == BINARY_TRUE_DIVIDE ||          \
@@ -200,6 +223,12 @@
                                  op == INPLACE_TRUE_DIVIDE)
 #else
 # define IS_NEW_DIVIDE(op)       0
+#endif
+
+#ifdef LIST_APPEND
+# define IS_LIST_APPEND(op)  (op == LIST_APPEND)
+#else
+# define IS_LIST_APPEND(op)   0
 #endif
  /***************************************************************/
 
@@ -211,6 +240,7 @@
 #define MP_HAS_J_MULTIPLE  0x10
 #define MP_LIGHT           0x20
 #define MP_IS_CTXDEP       0x40
+#define MP_IS_MODULE       0x80
 
 #define F(op)      ((IS_JUMP_INSTR(op)    ? MP_IS_JUMP        : 0) |    \
                     (HAS_JREL_INSTR(op)   ? MP_HAS_JREL       : 0) |    \
@@ -218,12 +248,13 @@
                     (HAS_J_MULTIPLE(op)   ? MP_HAS_J_MULTIPLE : 0) |    \
                     (IS_LIGHT_INSTR(op)   ? MP_LIGHT          : 0) |    \
                     (IS_CTXDEP_INSTR(op)  ? MP_IS_CTXDEP      : 0) |    \
+                    (IS_MODULE_INSTR(op)  ? MP_IS_MODULE      : 0) |    \
                     (OTHER_OPCODE(op)     ? MP_OTHER          : 0))
 
 /* opcode table -- the preprocessor expands this into several hundreds KB
    of code (which reduces down to 256 bytes!).  Hopefully not a problem
    for modern C compilers */
-static const char instr_control_flow[256] = {
+static const unsigned char instr_control_flow[256] = {
   F(0x00), F(0x01), F(0x02), F(0x03), F(0x04), F(0x05), F(0x06), F(0x07),
   F(0x08), F(0x09), F(0x0A), F(0x0B), F(0x0C), F(0x0D), F(0x0E), F(0x0F),
   F(0x10), F(0x11), F(0x12), F(0x13), F(0x14), F(0x15), F(0x16), F(0x17),
@@ -474,7 +505,7 @@ static void analyse_variables(struct instrnode_s* instrnodes,
 
 
 DEFINEFN
-PyObject* psyco_build_merge_points(PyCodeObject* co)
+PyObject* psyco_build_merge_points(PyCodeObject* co, int module)
 {
   PyObject* s;
   mergepoint_t* mp;
@@ -496,6 +527,20 @@ PyObject* psyco_build_merge_points(PyCodeObject* co)
       Py_INCREF(Py_None);
       return Py_None;
     }
+#ifdef CO_GENERATOR
+  /* check for this flag -- it is not enough to check for the presence of
+     a YIELD_VALUE instruction because such an instruction might exist but
+     be unreachable. */
+  if (co->co_flags & CO_GENERATOR)
+    {
+      debug_printf(1 + (strcmp(PyCodeObject_NAME(co), "?")==0),
+                   ("unsupported generator at %s\n",
+                    PyCodeObject_NAME(co)));
+      Py_INCREF(Py_None);
+      return Py_None;
+    }
+#endif  /* CO_GENERATOR */
+
   instrnodes = (struct instrnode_s*) PyMem_MALLOC(ibytes);
   if (instrnodes == NULL)
     OUT_OF_MEMORY();
@@ -588,6 +633,17 @@ PyObject* psyco_build_merge_points(PyCodeObject* co)
               valid_controlflow = false;
             }
           break;
+
+        case LOAD_GLOBAL:
+        case LOAD_NAME:
+          {
+            PyObject* namev = PyTuple_GET_ITEM(co->co_names, oparg);
+            char** p;
+            for (p = NoControlFlowIfBuiltin; *p; p++)
+              if (strcmp(PyString_AS_STRING(namev), *p) == 0)
+                valid_controlflow = false;
+          }
+          break;
         }
     }
   if (iblock != 0)
@@ -605,11 +661,11 @@ PyObject* psyco_build_merge_points(PyCodeObject* co)
             unsigned char op = instrnodes[i].opcode;
             int oparg = instrnodes[i+1].mask;
             int nextinstr = i+1 + instrnodes[i+1].back;
-            char flags = instr_control_flow[(int) op];
+            unsigned char flags = instr_control_flow[(int) op];
             if (flags == 0)
               if (op != COMPARE_OP || !SUPPORTED_COMPARE_ARG(oparg))
                 {
-                  /* unsupported instruction */
+                unsupported_instruction:
                   debug_printf(1 + (strcmp(PyCodeObject_NAME(co), "?")==0),
                                ("unsupported opcode %d at %s:%d\n",
                                 (int) op, PyCodeObject_NAME(co), i));
@@ -644,6 +700,12 @@ PyObject* psyco_build_merge_points(PyCodeObject* co)
                 instrnodes[nextinstr].inpaths = 99;
                 instrnodes[nextinstr].mp = MPSET_FORCE;
               }
+            if (flags & MP_IS_MODULE)
+              {
+                if (  /*!module*/  1)  // disabled, currently buggy
+                  goto unsupported_instruction;
+                mp_flags |= MP_FLAGS_MODULE;
+              }
             
             /* compact the next1-next2-next3 */
             if (instrnodes[i].next2 == NULL)
@@ -831,7 +893,10 @@ PyObject* psyco_build_merge_points(PyCodeObject* co)
   mp->bytecode_position = mp_flags;
 #if FULL_CONTROL_FLOW
   if (valid_controlflow)
-    analyse_variables(instrnodes, instrnodes+length, co);
+    {
+      mp->bytecode_position |= MP_FLAGS_CONTROLFLOW;
+      analyse_variables(instrnodes, instrnodes+length, co);
+    }
 #endif
   PyMem_FREE(instrnodes);
   return s;
@@ -866,7 +931,7 @@ mergepoint_t* psyco_next_merge_point(PyObject* mergepoints,
 }
 
 DEFINEFN
-PyObject* psyco_get_merge_points(PyCodeObject* co)
+PyObject* psyco_get_merge_points(PyCodeObject* co, int module)
 {
-  return PyCodeStats_MergePoints(PyCodeStats_Get(co));
+  return PyCodeStats_MergePoints(PyCodeStats_Get(co), module);
 }
