@@ -123,10 +123,15 @@ inline ceval_events_t* get_cevents(PyThreadState* tstate)
 }
 
 
+static PyObject* deleted_ceval_hook(PyFrameObject* frame, PyObject* arg)
+{
+	return NULL;
+}
+
 static void set_ceval_hook(ceval_events_t* cev, int when,
 			   ceval_event_fn fn, PyObject* arg)
 {
-	int n;
+	int n, i, allow;
 	struct cevents_s* events;
 	extra_assert(0 <= when && when < PyTrace_TOTAL);
 	events = cev->events + when;
@@ -136,13 +141,20 @@ static void set_ceval_hook(ceval_events_t* cev, int when,
 		OUT_OF_MEMORY();
 	events->items[n].fn = fn;
 	events->items[n].arg = arg;
-	Py_XINCREF(arg);
 	cev->events_total++;
-}
-
-static PyObject* deleted_ceval_hook(PyFrameObject* frame, PyObject* arg)
-{
-	return NULL;
+	if (arg != NULL) {
+		/* bound the total number of hooks by checking if there are
+		   too many other hooks with the same 'fn' */
+		allow = 8;
+		for (i=n; --i >= 0; ) {
+			if (events->items[i].fn == fn && !--allow) {
+				/* too many! remove an arbitrary one */
+				events->items[i].fn = &deleted_ceval_hook;
+				cev->events_total--;
+				break;
+			}
+		}
+	}
 }
 
 static void unset_ceval_hook(ceval_events_t* cev, int when,
@@ -164,6 +176,17 @@ static void unset_ceval_hook(ceval_events_t* cev, int when,
 	}
 }
 
+#if VERBOSE_STATS
+# define set_ceval_hook(cev, when, fn, arg)   do {                             \
+    stats_printf(("set_ceval_hook:   " #when ", " #fn ", " #arg " = 0x%x\n", (int)arg));  \
+    set_ceval_hook(cev, when, fn, arg);                                        \
+  } while (0)
+# define unset_ceval_hook(cev, when, fn, arg)   do {                           \
+    stats_printf(("unset_ceval_hook: " #when ", " #fn ", " #arg " = 0x%x\n", (int)arg));  \
+    unset_ceval_hook(cev, when, fn, arg);                                      \
+  } while (0)
+#endif
+
 inline bool call_ceval_hooks(ceval_events_t* cev, int what, PyFrameObject* f)
 {
 	bool r = true;
@@ -172,9 +195,10 @@ inline bool call_ceval_hooks(ceval_events_t* cev, int what, PyFrameObject* f)
 #if HAVE_DYN_COMPILE
 	PyObject* codebuf;
 #endif
-	PyObject* harg;
 	PyObject* obj;
-	extra_assert(0 <= what && what < PyTrace_TOTAL);
+	extra_assert(what >= 0);
+	if (what >= PyTrace_TOTAL)
+		return true;   /* Python >= 2.4 defines PyTrace_C_xxx */
 #if VERBOSE_LEVEL >= 3
         stats_printf(("hook: %d %s\n", what, PyCodeObject_NAME(f->f_code)));
 #endif
@@ -186,12 +210,9 @@ inline bool call_ceval_hooks(ceval_events_t* cev, int what, PyFrameObject* f)
 			return true;  /* done */
 		n--;
 		extra_assert(n < events->count);
-		harg = events->items[n].arg;
-		codebuf = events->items[n].fn(f, harg);
+		codebuf = events->items[n].fn(f, events->items[n].arg);
 		if (events->items[n].fn == &deleted_ceval_hook) {
-			extra_assert(harg == events->items[n].arg);
 			events->items[n] = events->items[--events->count];
-			Py_XDECREF(harg);
 		}
 	} while (codebuf == NULL);
 #endif
@@ -200,13 +221,10 @@ inline bool call_ceval_hooks(ceval_events_t* cev, int what, PyFrameObject* f)
 	while (n != 0) {
 		n--;
 		extra_assert(n < events->count);
-		harg = events->items[n].arg;
-		obj = events->items[n].fn(f, harg);
+		obj = events->items[n].fn(f, events->items[n].arg);
 		Py_XDECREF(obj);
 		if (events->items[n].fn == &deleted_ceval_hook) {
-			extra_assert(harg == events->items[n].arg);
 			events->items[n] = events->items[--events->count];
-			Py_XDECREF(harg);
 		}
 	}
 #if HAVE_DYN_COMPILE
@@ -496,16 +514,17 @@ static PyObject* profile_call(PyFrameObject* frame, PyObject* arg)
 			if (cs->st_codebuf == NULL) {
 				/* not already compiled, compile it now */
 				PyObject* g = frame->f_globals;
-				int rec;
+				int rec, module;
 				stats_printf(("stats: compile code:  %s\n",
 					      PyCodeObject_NAME(frame->f_code)));
 				if (PyInt_Check(cs->st_globals))
 					rec = PyInt_AS_LONG(cs->st_globals);
 				else
 					rec = DEFAULT_RECURSION;
+                                module = frame->f_globals == frame->f_locals;
 				cs->st_codebuf = PsycoCode_CompileCode(
 								frame->f_code,
-								g, rec);
+								g, rec, module);
 				if (cs->st_codebuf == Py_None)
 					g = NULL;  /* failed */
 				else {
@@ -562,15 +581,16 @@ static PyObject* do_fullcompile(PyFrameObject* frame, PyObject* arg)
 	if (cs->st_codebuf == NULL) {
 		/* not already compiled, compile it now */
 		PyObject* g = frame->f_globals;
-		int rec;
+		int rec, module;
 		stats_printf(("stats: full compile code:  %s\n",
 			      PyCodeObject_NAME(frame->f_code)));
 		if (cs->st_globals && PyInt_Check(cs->st_globals))
 			rec = PyInt_AS_LONG(cs->st_globals);
 		else
 			rec = DEFAULT_RECURSION;
+                module = frame->f_globals == frame->f_locals;
 		cs->st_codebuf = PsycoCode_CompileCode(frame->f_code,
-						       g, rec);
+						       g, rec, module);
 		if (cs->st_codebuf == Py_None)
 			g = NULL;  /* failed */
 		else {
